@@ -23,6 +23,7 @@
 #include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <sheet.h>
 #include <cell.h>
@@ -37,12 +38,17 @@
 #include <widgets/gnumeric-expr-entry.h>
 #include "simulation.h"
 
+#include <string.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkliststore.h>
 
 #define SIMULATION_KEY         "simulation-dialog"
 
 typedef GenericToolState SimulationState;
 
-static GtkTextBuffer   *log_buffer;
 static GtkTextBuffer   *results_buffer;
 static int             results_sim_index;
 static simulation_t    *current_sim;
@@ -55,17 +61,17 @@ static simulation_t    *current_sim;
  * Update the dialog widgets sensitivity
  **/
 static void
-simulation_update_sensitivity_cb (__attribute__((unused))GtkWidget *dummy,
-				       SimulationState *state)
+simulation_update_sensitivity_cb (G_GNUC_UNUSED GtkWidget *dummy,
+				  SimulationState *state)
 {
-        Value *output_range = NULL;
-        Value *input_range  = NULL;
-        Value *output_vars  = NULL;
+        GnmValue *output_range = NULL;
+        GnmValue *input_range  = NULL;
+        GnmValue *output_vars  = NULL;
 
 	int i;
 
         input_range = gnm_expr_entry_parse_as_value (
-		GNUMERIC_EXPR_ENTRY (state->input_entry), state->sheet);
+		GNM_EXPR_ENTRY (state->input_entry), state->sheet);
 	if (input_range == NULL) {
 		gtk_label_set_text (GTK_LABEL (state->warning),
 				    _("The input variable range is invalid."));
@@ -87,7 +93,7 @@ simulation_update_sensitivity_cb (__attribute__((unused))GtkWidget *dummy,
 	i = gnumeric_glade_group_value (state->gui, output_group);
 	if (i == 2) {
 		output_range = gnm_expr_entry_parse_as_value
-			(GNUMERIC_EXPR_ENTRY (state->output_entry),
+			(GNM_EXPR_ENTRY (state->output_entry),
 			 state->sheet);
 		if (output_range == NULL) {
 			gtk_label_set_text (GTK_LABEL (state->warning),
@@ -112,8 +118,8 @@ prepare_ranges (simulation_t *sim)
 	    sim->outputs->type != VALUE_CELLRANGE)
 		return TRUE;
 
-	sim->ref_inputs  = value_to_rangeref (sim->inputs, FALSE);
-	sim->ref_outputs = value_to_rangeref (sim->outputs, FALSE);
+	sim->ref_inputs  = rangeref_dup (value_get_rangeref (sim->inputs));
+	sim->ref_outputs = rangeref_dup (value_get_rangeref (sim->outputs));
 
 	sim->n_input_vars =
 		(abs (sim->ref_inputs->a.col - sim->ref_inputs->b.col) + 1) *
@@ -132,12 +138,8 @@ prepare_ranges (simulation_t *sim)
 		for (n = base_row;
 		     n<= MAX (sim->ref_inputs->a.row, sim->ref_inputs->b.row);
 		     n++) {
-			Cell *cell;
-
-			cell = sheet_cell_fetch (sim->ref_inputs->a.sheet,
-						 i, n);
-			sim->list_inputs = g_slist_append (sim->list_inputs,
-							   cell);
+			GnmCell *cell = sheet_cell_fetch (sim->ref_inputs->a.sheet, i, n);
+			sim->list_inputs = g_slist_append (sim->list_inputs, cell);
 		}
 	}
 
@@ -150,12 +152,8 @@ prepare_ranges (simulation_t *sim)
 		for (n = base_row;
 		     n<= MAX (sim->ref_outputs->a.row, sim->ref_outputs->b.row);
 		     n++) {
-			Cell *cell;
-
-			cell = sheet_cell_fetch (sim->ref_outputs->a.sheet,
-						 i, n);
-			sim->list_outputs = g_slist_append (sim->list_outputs,
-							    cell);
+			GnmCell *cell = sheet_cell_fetch (sim->ref_outputs->a.sheet, i, n);
+			sim->list_outputs = g_slist_append (sim->list_outputs, cell);
 		}
 	}
 
@@ -163,32 +161,76 @@ prepare_ranges (simulation_t *sim)
 }
 
 static void
-update_log (simulation_t *sim)
+update_log (SimulationState *state, simulation_t *sim)
 {
-	const gchar   *t1 = _("Simulations\t\t\t= ");
-	const gchar   *t2 = _("Iterations\t\t\t= ");
-	const gchar   *t3 = _("# Input variables\t\t= ");
-	const gchar   *t4 = _("# Output variables\t= ");
-	const gchar   *t5 = _("Runtime\t\t\t= ");
-	const gchar   *t6 = _("Run on\t\t\t= ");
-	GString *buf;
+	const gchar   *txt [6] = {
+		_("Simulations"), _("Iterations"), _("# Input variables"),
+		_("# Output variables"), _("Runtime"), _("Run on")
+	};
+	GtkTreeIter  iter;
+	GtkListStore *store;
+	GtkTreePath  *path;
+	GtkWidget    *view;
+	GString      *buf;
+	int          i;
 
-	buf = g_string_new ("");
-	g_string_append_printf (buf, "%s %d\n", t1,
-			   sim->last_round - sim->first_round + 1);
-	g_string_append_printf (buf, "%s %d\n", t2, sim->n_iterations);
-	g_string_append_printf (buf, "%s %d\n", t3, sim->n_input_vars);
-	g_string_append_printf (buf, "%s %d\n", t4, sim->n_output_vars);
-	g_string_append_printf (buf, "%s %.2f sec.\n", t5, 
-			   sim->end.tv_sec - sim->start.tv_sec +
-			   (sim->end.tv_usec - sim->start.tv_usec) /
-			   (gnum_float) G_USEC_PER_SEC);
-	g_string_append_printf (buf, "%s ", t6);
-	dao_append_date (buf);
-	g_string_append_printf (buf, "\n");
+	view = glade_xml_get_widget (state->gui, "last-run-view");
 
-	gtk_text_buffer_set_text (log_buffer, buf->str, strlen (buf->str));
-	g_string_free (buf, FALSE);
+	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+
+	for (i = 0; i < 6; i++) {
+		buf = g_string_new (NULL);
+		switch (i) {
+		case 0:
+			g_string_append_printf (buf, "%d",
+						sim->last_round -
+						sim->first_round + 1);
+			break;
+		case 1:
+			g_string_append_printf (buf, "%d", sim->n_iterations);
+			break;
+		case 2:
+			g_string_append_printf (buf, "%d", sim->n_input_vars);
+			break;
+		case 3:
+			g_string_append_printf (buf, "%d", sim->n_output_vars);
+			break;
+		case 4:
+			g_string_append_printf (buf, "%.2g",
+						sim->end.tv_sec -
+						sim->start.tv_sec +
+						(sim->end.tv_usec -
+						 sim->start.tv_usec) /
+						(gnm_float) G_USEC_PER_SEC);
+			break;
+		case 5:
+			dao_append_date (buf);
+			break;
+		default:
+			g_string_append_printf (buf, "Error");
+			break;
+		}
+
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, txt [i], 1, buf->str, -1);
+		g_string_free (buf, FALSE);
+	}
+
+	path = gtk_tree_path_new_from_string ("0");
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
+	gtk_tree_path_free (path);
+
+	gtk_tree_view_append_column
+		(GTK_TREE_VIEW (view),
+		 gtk_tree_view_column_new_with_attributes
+		 (_("Name"),
+		  gtk_cell_renderer_text_new (), "text", 0, NULL));
+	gtk_tree_view_append_column
+		(GTK_TREE_VIEW (view),
+		 gtk_tree_view_column_new_with_attributes
+		 (_("Value"),
+		  gtk_cell_renderer_text_new (), "text", 1, NULL));
+	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (store));
 }
 
 static void
@@ -197,7 +239,7 @@ update_results_view (simulation_t *sim)
 	GString *buf;
 	int     i;
 
-	buf = g_string_new ("");
+	buf = g_string_new (NULL);
 
 	g_string_append_printf (buf, "Simulation #%d\n\n", results_sim_index + 1);
 	g_string_append_printf (buf, "%-20s %10s %10s %10s\n", _("Variable"),
@@ -214,7 +256,7 @@ update_results_view (simulation_t *sim)
 }
 
 static void
-prev_button_cb (__attribute__((unused))GtkWidget *button,
+prev_button_cb (G_GNUC_UNUSED GtkWidget *button,
 		SimulationState *state)
 {
 	GtkWidget *w;
@@ -233,7 +275,7 @@ prev_button_cb (__attribute__((unused))GtkWidget *button,
 }
 
 static void
-next_button_cb (__attribute__((unused))GtkWidget *button,
+next_button_cb (G_GNUC_UNUSED GtkWidget *button,
 		SimulationState *state)
 {
 	GtkWidget *w;
@@ -261,7 +303,7 @@ next_button_cb (__attribute__((unused))GtkWidget *button,
  * contain sensible data.
  **/
 static void
-simulation_ok_clicked_cb (__attribute__((unused)) GtkWidget *button,
+simulation_ok_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 			  SimulationState *state)
 {
 	data_analysis_output_t  dao;
@@ -272,7 +314,7 @@ simulation_ok_clicked_cb (__attribute__((unused)) GtkWidget *button,
 	simulation_tool_destroy (current_sim);
 
 	sim.inputs = gnm_expr_entry_parse_as_value
-		(GNUMERIC_EXPR_ENTRY (state->input_entry), state->sheet);
+		(GNM_EXPR_ENTRY (state->input_entry), state->sheet);
 
 	sim.outputs = gnm_expr_entry_parse_as_value
 		(state->input_entry_2, state->sheet);
@@ -298,9 +340,12 @@ simulation_ok_clicked_cb (__attribute__((unused)) GtkWidget *button,
 				   "equal than the number of the last round.");
 		goto out;
 	}
-		
+
 	current_sim = &sim;
 
+	w = glade_xml_get_widget (state->gui, "max-time");
+	sim.max_time = gtk_spin_button_get_value (GTK_SPIN_BUTTON (w)) - 1;
+
 	g_get_current_time (&sim.start);
 	err = simulation_tool (WORKBOOK_CONTROL (state->wbcg),
 			       &dao, &sim);
@@ -308,7 +353,7 @@ simulation_ok_clicked_cb (__attribute__((unused)) GtkWidget *button,
 
       	if (err == NULL) {
 		results_sim_index = sim.first_round;
-		update_log (&sim);
+		update_log (state, &sim);
 		update_results_view (&sim);
 
 		if (sim.last_round > results_sim_index) {
@@ -335,26 +380,13 @@ simulation_ok_clicked_cb (__attribute__((unused)) GtkWidget *button,
  * Close (destroy) the dialog
  **/
 static void
-cb_tool_cancel_clicked (__attribute__((unused))GtkWidget *button,
+cb_tool_cancel_clicked (G_GNUC_UNUSED GtkWidget *button,
 			GenericToolState *state)
 {
 	simulation_tool_destroy (current_sim);
 	gtk_widget_destroy (state->dialog);
 }
 
-static void
-init_log (SimulationState *state)
-{
-	GtkTextView     *view;
-	GtkTextTagTable *tag_table;
-
-	tag_table  = gtk_text_tag_table_new ();
-	log_buffer = gtk_text_buffer_new (tag_table);
-	view = GTK_TEXT_VIEW (glade_xml_get_widget (state->gui,
-						    "last-run-view"));
-	gtk_text_view_set_buffer (view, log_buffer);
-}
-
 static void
 init_results_view (SimulationState *state)
 {
@@ -377,7 +409,7 @@ init_results_view (SimulationState *state)
  *
  **/
 void
-dialog_simulation (WorkbookControlGUI *wbcg, __attribute__((unused))Sheet *sheet)
+dialog_simulation (WorkbookControlGUI *wbcg, G_GNUC_UNUSED Sheet *sheet)
 {
         SimulationState *state;
 	WorkbookControl *wbc;
@@ -393,7 +425,7 @@ dialog_simulation (WorkbookControlGUI *wbcg, __attribute__((unused))Sheet *sheet
 
 	state = g_new (SimulationState, 1);
 	if (dialog_tool_init (state, wbcg, wb_control_cur_sheet (wbc),
-			      "simulation.html",
+			      GNUMERIC_HELP_LINK_SIMULATION,
 			      "simulation.glade", "Simulation",
 			      _("_Inputs:"), _("_Outputs:"),
 			      _("Could not create the Simulation dialog."),
@@ -404,7 +436,6 @@ dialog_simulation (WorkbookControlGUI *wbcg, __attribute__((unused))Sheet *sheet
 			      0))
 		return;
 
-	init_log (state);
 	init_results_view (state);
 	current_sim = NULL;
 
