@@ -23,6 +23,7 @@
 #include "mathfunc.h"
 
 #include <pango/pangoft2.h>
+#include <gtk/gtkmain.h>
 #include <string.h>
 
 #undef DEBUG_REF_COUNT
@@ -74,7 +75,7 @@ get_substitute_font (gchar const *fontname)
 }
 
 static int
-style_font_string_width (StyleFont const *font, char const *str)
+style_font_string_width (GnmFont const *font, char const *str)
 {
 	int w;
 	pango_layout_set_text (font->pango.layout, str, -1);
@@ -83,7 +84,7 @@ style_font_string_width (StyleFont const *font, char const *str)
 }
 
 static double
-calc_font_width (StyleFont const *font, char const *teststr)
+calc_font_width (GnmFont const *font, char const *teststr)
 {
 	char const *p1, *p2;
 	int w = 0, w1, w2, dw;
@@ -111,13 +112,13 @@ calc_font_width (StyleFont const *font, char const *teststr)
 }
 
 
-static StyleFont *
+static GnmFont *
 style_font_new_simple (PangoContext *context,
 		       char const *font_name, double size_pts, double scale,
 		       gboolean bold, gboolean italic)
 {
-	StyleFont *font;
-	StyleFont key;
+	GnmFont *font;
+	GnmFont key;
 	int height;
 
 	if (font_name == NULL) {
@@ -136,7 +137,7 @@ style_font_new_simple (PangoContext *context,
 	key.is_italic = italic;
 	key.scale     = scale;
 
-	font = (StyleFont *) g_hash_table_lookup (style_font_hash, &key);
+	font = (GnmFont *) g_hash_table_lookup (style_font_hash, &key);
 	if (font == NULL) {
 		PangoFontDescription *desc;
 		double pts_scale;
@@ -144,7 +145,7 @@ style_font_new_simple (PangoContext *context,
 		if (g_hash_table_lookup (style_font_negative_hash, &key))
 			return NULL;
 
-		font = g_new0 (StyleFont, 1);
+		font = g_new0 (GnmFont, 1);
 		font->font_name = g_strdup (font_name);
 		font->size_pts  = size_pts;
 		font->scale     = scale;
@@ -208,7 +209,7 @@ style_font_new_simple (PangoContext *context,
 		font->approx_width.pixels.E = calc_font_width (font, "E");
 		font->approx_width.pixels.e = calc_font_width (font, "e");
 
-		pts_scale = application_display_dpi_get (TRUE) / 72.0;
+		pts_scale = gnm_app_display_dpi_get (TRUE) / 72.0;
 		font->approx_width.pts.digit =
 			font->approx_width.pixels.digit / pts_scale;
 		font->approx_width.pts.decimal =
@@ -234,12 +235,12 @@ style_font_new_simple (PangoContext *context,
 	return font;
 }
 
-StyleFont *
+GnmFont *
 style_font_new (PangoContext *context,
 		char const *font_name, double size_pts, double scale,
 		gboolean bold, gboolean italic)
 {
-	StyleFont *font;
+	GnmFont *font;
 
 	g_return_val_if_fail (font_name != NULL, NULL);
 	g_return_val_if_fail (size_pts > 0, NULL);
@@ -277,7 +278,7 @@ style_font_new (PangoContext *context,
 }
 
 void
-style_font_ref (StyleFont *sf)
+style_font_ref (GnmFont *sf)
 {
 	g_return_if_fail (sf != NULL);
 
@@ -292,7 +293,7 @@ style_font_ref (StyleFont *sf)
 }
 
 void
-style_font_unref (StyleFont *sf)
+style_font_unref (GnmFont *sf)
 {
 	g_return_if_fail (sf != NULL);
 	g_return_if_fail (sf->ref_count > 0);
@@ -343,8 +344,8 @@ style_font_unref (StyleFont *sf)
 gint
 style_font_equal (gconstpointer v, gconstpointer v2)
 {
-	StyleFont const *k1 = (StyleFont const *) v;
-	StyleFont const *k2 = (StyleFont const *) v2;
+	GnmFont const *k1 = (GnmFont const *) v;
+	GnmFont const *k2 = (GnmFont const *) v2;
 
 	if (k1->size_pts != k2->size_pts)
 		return 0;
@@ -362,7 +363,7 @@ style_font_equal (gconstpointer v, gconstpointer v2)
 guint
 style_font_hash_func (gconstpointer v)
 {
-	StyleFont const *k = (StyleFont const *) v;
+	GnmFont const *k = (GnmFont const *) v;
 
 	return k->size_pts + g_str_hash (k->font_name);
 }
@@ -406,9 +407,9 @@ gnm_pango_context_get (void)
 static void
 font_init (void)
 {
-	GConfClient *client = application_get_gconf_client ();
+	GConfClient *client = gnm_app_get_gconf_client ();
 	PangoContext *context;
-	StyleFont *gnumeric_default_font = NULL;
+	GnmFont *gnumeric_default_font = NULL;
 	int n_families, i;
 
 	gnumeric_default_font_name =
@@ -509,14 +510,14 @@ style_init (void)
 }
 
 static void
-delete_neg_font (StyleFont *sf, gpointer value, gpointer user_data)
+delete_neg_font (GnmFont *sf, gpointer value, gpointer user_data)
 {
 	g_free (sf->font_name);
 	g_free (sf);
 }
 
 static void
-list_cached_fonts (StyleFont *font, gpointer value, GSList **lp)
+list_cached_fonts (GnmFont *font, gpointer value, GSList **lp)
 {
 	*lp = g_slist_prepend (*lp, font);
 }
@@ -534,7 +535,7 @@ style_shutdown (void)
 		GSList *fonts = NULL, *tmp;
 		g_hash_table_foreach (style_font_hash, (GHFunc) list_cached_fonts, &fonts);
 		for (tmp = fonts; tmp; tmp = tmp->next) {
-			StyleFont *sf = tmp->data;
+			GnmFont *sf = tmp->data;
 			if (sf->ref_count != 1)
 				g_warning ("Font %s has %d references instead of the expected single.",
 					   sf->font_name, sf->ref_count);
@@ -557,21 +558,24 @@ style_shutdown (void)
  * What changes are required after applying the supplied style.
  */
 SpanCalcFlags
-required_updates_for_style (MStyle *style)
+required_updates_for_style (GnmStyle *style)
 {
 	gboolean const size_change =
-	    (mstyle_is_element_set  (style, MSTYLE_FONT_NAME) ||
-	     mstyle_is_element_set  (style, MSTYLE_FONT_BOLD) ||
-	     mstyle_is_element_set  (style, MSTYLE_FONT_ITALIC) ||
-	     mstyle_is_element_set  (style, MSTYLE_FONT_SIZE) ||
-	     mstyle_is_element_set  (style, MSTYLE_WRAP_TEXT));
+	    (mstyle_is_element_set (style, MSTYLE_FONT_NAME) ||
+	     mstyle_is_element_set (style, MSTYLE_FONT_BOLD) ||
+	     mstyle_is_element_set (style, MSTYLE_FONT_ITALIC) ||
+	     mstyle_is_element_set (style, MSTYLE_FONT_SIZE) ||
+	     mstyle_is_element_set (style, MSTYLE_WRAP_TEXT));
 	gboolean const format_change =
 	    (mstyle_is_element_set (style, MSTYLE_FORMAT) ||
-	     mstyle_is_element_set (style, MSTYLE_INDENT));
+	     mstyle_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH) ||
+	     mstyle_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
+	     mstyle_is_element_set (style, MSTYLE_INDENT) ||
+	     mstyle_is_element_set (style, MSTYLE_ALIGN_H) ||
+	     mstyle_is_element_set (style, MSTYLE_COLOR_FORE));
 
-	return format_change
-	    ? SPANCALC_RE_RENDER|SPANCALC_RESIZE
-	    : size_change ? SPANCALC_RESIZE
+	return (format_change|size_change)
+		? (SPANCALC_RE_RENDER|SPANCALC_RESIZE)
 			  : SPANCALC_SIMPLE;
 }
 
@@ -584,7 +588,7 @@ required_updates_for_style (MStyle *style)
  * value.
  */
 StyleHAlignFlags
-style_default_halign (MStyle const *mstyle, Cell const *c)
+style_default_halign (GnmStyle const *mstyle, GnmCell const *c)
 {
 	StyleHAlignFlags align = mstyle_get_align_h (mstyle);
 
@@ -600,6 +604,7 @@ style_default_halign (MStyle const *mstyle, Cell const *c)
 		for (v = c->value; v != NULL ; )
 			switch (v->type) {
 			case VALUE_BOOLEAN :
+			case VALUE_ERROR :
 				return HALIGN_CENTER;
 
 			case VALUE_INTEGER :
