@@ -2,18 +2,18 @@
 #ifndef _GNM_COMBO_TEXT_H
 #define _GNM_COMBO_TEXT_H
 
-#include <widgets/gtk-combo-box.h>
+#include <widgets/gnm-combo-box.h>
 
 G_BEGIN_DECLS
 
 #define GNM_TYPE_COMBO_TEXT	(gnm_combo_text_get_type ())
 #define GNM_COMBO_TEXT(obj)	(G_TYPE_CHECK_INSTANCE_CAST (obj, GNM_TYPE_COMBO_TEXT, GnmComboText))
-#define GNM_IS_COMBO_TEXT(obj)	(G_TYPE_CHECK_INSTANCE_TYPE (obj, GNM_TYPE_COMBO_TEXT))
+#define IS_GNM_COMBO_TEXT(obj)	(G_TYPE_CHECK_INSTANCE_TYPE (obj, GNM_TYPE_COMBO_TEXT))
 
 typedef struct _GnmComboText	   GnmComboText;
 
 struct _GnmComboText {
-	GtkComboBox parent;
+	GnmComboBox parent;
 
 	GCompareFunc cmp_func;
 
