@@ -3,7 +3,7 @@
  * workbook-edit.c: Keeps track of the cell editing process.
  *
  * Author:
- *   Miguel de Icaza (miguel@helixcode.com)
+ *   Miguel de Icaza (miguel@ximian.com)
  *   Jody Goldberg (jody@gnome.org)
  *
  * (C) 2000-2001 Ximian, Inc.
@@ -21,7 +21,7 @@
 #include "commands.h"
 #include "gnumeric-gconf.h"
 #include "mstyle.h"
-#include "sheet-control-gui.h"
+#include "sheet-control-gui-priv.h"
 #include "sheet-style.h"
 #include "sheet-view.h"
 #include "sheet.h"
@@ -34,7 +34,6 @@
 #include "widgets/gnumeric-expr-entry.h"
 
 #include <gtk/gtk.h>
-#include <ctype.h>
 #include <string.h>
 
 /*
@@ -43,10 +42,8 @@
 void
 wbcg_auto_complete_destroy (WorkbookControlGUI *wbcg)
 {
-	if (wbcg->auto_complete_text){
-		g_free (wbcg->auto_complete_text);
-		wbcg->auto_complete_text = NULL;
-	}
+	g_free (wbcg->auto_complete_text);
+	wbcg->auto_complete_text = NULL;
 
 	if (wbcg->edit_line.signal_changed >= 0) {
 		g_signal_handler_disconnect (GTK_OBJECT (wbcg_get_entry (wbcg)),
@@ -63,15 +60,17 @@ wbcg_auto_complete_destroy (WorkbookControlGUI *wbcg)
 
 }
 
-static gboolean
-wbcg_edit_error_dialog (WorkbookControlGUI *wbcg, char *str)
-{
-	return wb_control_validation_msg (WORKBOOK_CONTROL (wbcg),
-		VALIDATION_STYLE_STOP, NULL, str) != 1;
-}
-
+/**
+ * wbcg_edit_finish :
+ * @wbcg : #WorkbookControlGUI
+ * @accept : Should any changes be accepted or discarded
+ * @showed_dialog : If non-NULL will indicate if a dialog was displayed.
+ *
+ * Return TRUE if editing completed successfully, or we were no editing.
+ **/
 gboolean
-wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
+wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept,
+		  gboolean *showed_dialog)
 {
 	Sheet *sheet;
 	SheetView *sv;
@@ -85,6 +84,9 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 
 	wbcg_focus_cur_scg (wbcg);
 
+	if (showed_dialog != NULL)
+		*showed_dialog = FALSE;
+
 	/* Remove the range selection cursor if it exists */
 	if (NULL != wbcg->rangesel)
 		scg_rangesel_stop (wbcg->rangesel, !accept);
@@ -94,8 +96,11 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 		 * Do NOT remove until later it if we are editing, it is possible
 		 * that we may want to continue editing.
 		 */
-		if (wbcg->edit_line.guru != NULL)
-			gtk_widget_destroy (wbcg->edit_line.guru);
+		if (wbcg->edit_line.guru != NULL) {
+			GtkWidget *w = wbcg->edit_line.guru;
+			wbcg_edit_detach_guru (wbcg);
+			gtk_widget_destroy (w);
+		}
 
 		return TRUE;
 	}
@@ -110,34 +115,38 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 		ValidationStatus valid;
 		char *free_txt = NULL;
 		char const *txt = wbcg_edit_get_display_text (wbcg);
-		MStyle *mstyle = sheet_style_get (sheet, sv->edit_pos.col, sv->edit_pos.row);
+		GnmStyle *mstyle = sheet_style_get (sheet, sv->edit_pos.col, sv->edit_pos.row);
 		char const *expr_txt = NULL;
 
 		/* BE CAREFUL the standard fmts must not NOT include '@' */
-		Value *value = format_match (txt, mstyle_get_format (mstyle));
+		GnmValue *value = format_match (txt, mstyle_get_format (mstyle),
+					     workbook_date_conv (sheet->workbook));
 		if (value != NULL)
 			value_release (value);
 		else
 			expr_txt = gnm_expr_char_start_p (txt);
 
-		if (expr_txt != NULL && *expr_txt != '\0') {
+		/* NOTE : do not modify gnm_expr_char_start_p to exclude "-"
+		 * it _can_ start an expression, which is required for rangesel
+		 * it just isn't an expression. */
+		if (expr_txt != NULL && *expr_txt != '\0' && strcmp (expr_txt, "-")) {
 			GnmExpr const *expr = NULL;
-			ParsePos    pp;
-			ParseError  perr;
+			GnmParsePos    pp;
+			GnmParseError  perr;
 
 			parse_pos_init_editpos (&pp, sv);
 			parse_error_init (&perr);
 			expr = gnm_expr_parse_str (expr_txt,
-				&pp, GNM_EXPR_PARSE_DEFAULT, &rangeref_parse, &perr);
+				&pp, GNM_EXPR_PARSE_DEFAULT, gnm_expr_conventions_default, &perr);
 			/* Try adding a single extra closing paren to see if it helps */
 			if (expr == NULL && perr.err != NULL &&
 			    perr.err->code == PERR_MISSING_PAREN_CLOSE) {
-				ParseError tmp_err;
+				GnmParseError tmp_err;
 				char *tmp = g_strconcat (txt, ")", NULL);
 				parse_error_init (&tmp_err);
 				expr = gnm_expr_parse_str (gnm_expr_char_start_p (tmp),
 					&pp, GNM_EXPR_PARSE_DEFAULT,
-					&rangeref_parse, &tmp_err);
+					gnm_expr_conventions_default, &tmp_err);
 				parse_error_free (&tmp_err);
 
 				if (expr != NULL)
@@ -147,7 +156,7 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 			}
 
 			if (expr == NULL && perr.err != NULL) {
-				int reedit;
+				ValidationStatus reedit;
 
 				/* set focus _before_ selection.  gtk2 seems to
 				 * screw with selection in gtk_entry_grab_focus
@@ -157,17 +166,26 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 
 				if (perr.begin_char != 0 || perr.end_char != 0) {
 					int offset = expr_txt - txt;
-					gtk_entry_select_region (wbcg_get_entry (wbcg),
+					gtk_editable_select_region (GTK_EDITABLE (wbcg_get_entry (wbcg)),
 						offset + perr.begin_char,
 						offset + perr.end_char);
 				} else
 					gtk_editable_set_position (
 						GTK_EDITABLE (wbcg_get_entry (wbcg)), -1);
 
-				reedit = wbcg_edit_error_dialog (wbcg, perr.err->message);
+				reedit = wb_control_validation_msg (WORKBOOK_CONTROL (wbcg),
+					VALIDATION_STYLE_PARSE_ERROR, NULL, perr.err->message);
+				if (showed_dialog != NULL)
+					*showed_dialog = TRUE;
+
 				parse_error_free (&perr);
-				if (reedit)
+				if (reedit == VALIDATION_STATUS_INVALID_EDIT)
 					return FALSE;
+				/* restore focus to sheet , or we'll leave edit
+				 * mode only to jump right back in in the new
+				 * cell because it looks like someone just
+				 * focused on the edit line (eg hit F2) */
+				wbcg_focus_cur_scg (wbcg);
 			}
 			if (expr != NULL)
 				gnm_expr_unref (expr);
@@ -177,7 +195,8 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 		 * a validation condition depends on the new value.
 		 */
 		cmd_set_text (wbc, sheet, &sv->edit_pos, txt);
-		valid = validation_eval (wbc, mstyle, sheet, &sv->edit_pos);
+		valid = validation_eval (wbc, mstyle, sheet, &sv->edit_pos,
+					 showed_dialog);
 
 		if (free_txt != NULL)
 			g_free (free_txt);
@@ -195,7 +214,7 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 	} else {
 		if (sv == wb_control_cur_sheet_view (wbc)) {
 			/* Redraw the cell contents in case there was a span */
-			Range tmp; tmp.start = tmp.end = sv->edit_pos;
+			GnmRange tmp; tmp.start = tmp.end = sv->edit_pos;
 			sheet_range_bounding_box (sv->sheet, &tmp);
 			sv_redraw_range (wb_control_cur_sheet_view (wbc), &tmp);
 		}
@@ -211,7 +230,7 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, gboolean accept)
 
 	if (wbcg->edit_line.guru != NULL) {
 		GtkWidget *w = wbcg->edit_line.guru;
-		wbcg->edit_line.guru = NULL;
+		wbcg_edit_detach_guru (wbcg);
 		gtk_widget_destroy (w);
 	}
 	wb_control_edit_set_sensitive (wbc, FALSE, TRUE);
@@ -238,11 +257,14 @@ static void
 workbook_edit_complete_notify (char const *text, void *closure)
 {
 	WorkbookControlGUI *wbcg = closure;
+	SheetControlGUI    *scg  = wbcg_cur_scg (wbcg);
 
-	if (wbcg->auto_complete_text)
-		g_free (wbcg->auto_complete_text);
-
+	g_free (wbcg->auto_complete_text);
 	wbcg->auto_complete_text = g_strdup (text);
+
+	SCG_FOREACH_PANE (scg, pane,
+		if (pane->editor != NULL)
+			foo_canvas_item_request_update (FOO_CANVAS_ITEM (pane->editor)););
 }
 
 static void
@@ -264,7 +286,7 @@ entry_changed (GtkEntry *entry, void *data)
 	 * does not begin with an alphabetic character.
 	 */
 	if (text_len < wbcg->auto_max_size ||
-	    !isalpha((unsigned char)*text))
+	    !g_unichar_isalpha (g_utf8_get_char (text)))
 		wbcg->auto_completing = FALSE;
 
 	if (wbv->do_auto_completion && wbcg->auto_completing)
@@ -296,7 +318,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 	static gboolean inside_editing = FALSE;
 	SheetView *sv;
 	SheetControlGUI *scg;
-	Cell *cell;
+	GnmCell *cell;
 	char *text = NULL;
 	int col, row;
 	WorkbookView *wbv;
@@ -319,7 +341,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 	row = sv->edit_pos.row;
 
 	/* don't edit a locked cell */
-	/* TODO : extend this to disable edits that can not succeed
+	/* TODO : extend this to disable edits that cannot succeed
 	 * like editing a single cell of an array.  I think we have enough
 	 * information if we look at the selection.
 	 */
@@ -327,7 +349,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 	    mstyle_get_content_locked (sheet_style_get (sv->sheet, col, row))) {
 		char *pos =  g_strdup_printf ( _("%s!%s is locked"),
 			sv->sheet->name_quoted, cell_coord_name (col, row));
-		gnumeric_error_invalid (COMMAND_CONTEXT (wbcg), pos,
+		gnm_cmd_context_error_invalid (GNM_CMD_CONTEXT (wbcg), pos,
 			wb_view_is_protected (wbv, FALSE)
 			 ? _("Unprotect the workbook to enable editing.")
 			 : _("Unprotect the sheet to enable editing."));
@@ -336,7 +358,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 		return FALSE;
 	}
 
-	application_clipboard_unant ();
+	gnm_app_clipboard_unant ();
 	wb_control_edit_set_sensitive (WORKBOOK_CONTROL (wbcg), TRUE, FALSE);
 
 	cell = sheet_cell_get (sv->sheet, col, row);
@@ -355,8 +377,8 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 
 	gnm_expr_entry_set_scg (wbcg->edit_line.entry, scg);
 	gnm_expr_entry_set_flags (wbcg->edit_line.entry,
-		GNUM_EE_SHEET_OPTIONAL,
-		GNUM_EE_SINGLE_RANGE | GNUM_EE_SHEET_OPTIONAL);
+		GNM_EE_SHEET_OPTIONAL | GNM_EE_FORMULA_ONLY,
+		GNM_EE_SINGLE_RANGE | GNM_EE_SHEET_OPTIONAL | GNM_EE_FORMULA_ONLY);
 	scg_edit_start (scg);
 
 	/* Redraw the cell contents in case there was a span */
@@ -364,7 +386,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 
 	/* Activate auto-completion if this is not an expression */
 	if (wbv->do_auto_completion &&
-	    (text == NULL || isalpha ((unsigned char)*text))) {
+	    (text == NULL || g_unichar_isalpha (g_utf8_get_char (text)))) {
 		wbcg->auto_complete = complete_sheet_new (
 			sv->sheet, col, row,
 			workbook_edit_complete_notify, wbcg);
@@ -374,9 +396,14 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 		wbcg->auto_complete = NULL;
 
 	/* Give the focus to the edit line */
-	if (!cursorp)
-		gtk_window_set_focus (GTK_WINDOW (wbcg->toplevel),
-				      GTK_WIDGET (wbcg_get_entry (wbcg)));
+	if (!cursorp) {
+		GtkEntry *w = wbcg_get_entry (wbcg);
+		gtk_window_set_focus (GTK_WINDOW (wbcg->toplevel), GTK_WIDGET (w));
+		/* the gtk-entry-select-on-focus property of GtkEntry is not
+		 * what we want here.  Rather than trying some gtk-2.2 specific
+		 * magic just undo the selection change here */
+		gtk_editable_select_region (GTK_EDITABLE (w), -1, -1);
+	}
 
 	wbcg->wb_control.editing = TRUE;
 	wbcg->wb_control.editing_sheet = sv->sheet;
@@ -406,7 +433,7 @@ wbcg_get_entry (WorkbookControlGUI const *wbcg)
 	return gnm_expr_entry_get_entry (wbcg->edit_line.entry);
 }
 
-GnumericExprEntry *
+GnmExprEntry *
 wbcg_get_entry_logical (WorkbookControlGUI const *wbcg)
 {
 	g_return_val_if_fail (wbcg != NULL, NULL);
@@ -418,13 +445,13 @@ wbcg_get_entry_logical (WorkbookControlGUI const *wbcg)
 }
 
 void
-wbcg_set_entry (WorkbookControlGUI *wbcg, GnumericExprEntry *entry)
+wbcg_set_entry (WorkbookControlGUI *wbcg, GnmExprEntry *entry)
 {
 	g_return_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg));
 
 	if (wbcg->edit_line.temp_entry != entry) {
-		wbcg->edit_line.temp_entry = entry;
 		scg_rangesel_stop (wbcg_cur_scg (wbcg), FALSE);
+		wbcg->edit_line.temp_entry = entry;
 	}
 }
 
@@ -440,10 +467,13 @@ wbcg_edit_attach_guru_main (WorkbookControlGUI *wbcg, GtkWidget *guru)
 	/* Make sure we don't have anything anted.
 	 * this protects against two anted regions showing up
 	 */
-	application_clipboard_unant ();
+	gnm_app_clipboard_unant ();
+
+	/* don't sit end 'End' mode when a dialog comes up */
+	wbcg_set_end_mode (wbcg, FALSE);
 
 	wbcg->edit_line.guru = guru;
-	gtk_entry_set_editable (wbcg_get_entry (wbcg), FALSE);
+	gtk_editable_set_editable (GTK_EDITABLE (wbcg_get_entry (wbcg)), FALSE);
 	wb_control_edit_set_sensitive (wbc, FALSE, FALSE);
 	wb_control_menu_state_update (wbc, MS_GURU_MENU_ITEMS);
 
@@ -453,9 +483,9 @@ static void
 cb_guru_set_focus (GtkWidget *window, GtkWidget *focus_widget,
 		   WorkbookControlGUI *wbcg)
 {
-	GnumericExprEntry *gee = NULL;
-	if (focus_widget != NULL && IS_GNUMERIC_EXPR_ENTRY (focus_widget->parent))
-		gee = GNUMERIC_EXPR_ENTRY (focus_widget->parent);
+	GnmExprEntry *gee = NULL;
+	if (focus_widget != NULL && IS_GNM_EXPR_ENTRY (focus_widget->parent))
+		gee = GNM_EXPR_ENTRY (focus_widget->parent);
 	wbcg_set_entry (wbcg, gee);
 }
 
@@ -473,14 +503,14 @@ wbcg_edit_attach_guru (WorkbookControlGUI *wbcg, GtkWidget *guru)
 }
 
 void
-wbcg_edit_attach_guru_with_unfocused_rs (WorkbookControlGUI *wbcg, GtkWidget *guru, 
-					 GnumericExprEntry *gee)
+wbcg_edit_attach_guru_with_unfocused_rs (WorkbookControlGUI *wbcg, GtkWidget *guru,
+					 GnmExprEntry *gee)
 {
 	g_return_if_fail (guru != NULL);
 	g_return_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg));
 
 	wbcg_edit_attach_guru_main (wbcg, guru);
-	
+
 	if (gnm_app_prefs->unfocused_range_selection) {
 		if (gee)
 			wbcg_set_entry (wbcg, gee);
@@ -496,12 +526,14 @@ wbcg_edit_detach_guru (WorkbookControlGUI *wbcg)
 	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
 	g_return_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg));
 
+	/* don't sit end 'End' mode when a dialog comes up */
+	wbcg_set_end_mode (wbcg, FALSE);
 	if (wbcg->edit_line.guru == NULL)
 		return;
 
 	wbcg_set_entry (wbcg, NULL);
 	wbcg->edit_line.guru = NULL;
-	gtk_entry_set_editable (wbcg_get_entry (wbcg), TRUE);
+	gtk_editable_set_editable (GTK_EDITABLE (wbcg_get_entry (wbcg)), TRUE);
 	wb_control_edit_set_sensitive (wbc, FALSE, TRUE);
 	wb_control_menu_state_update (wbc, MS_GURU_MENU_ITEMS);
 }
@@ -551,7 +583,7 @@ auto_complete_matches (WorkbookControlGUI *wbcg)
 			perform_test = TRUE;
 
 		if (perform_test)
-			if (cursor_pos != (int)strlen (text))
+			if (cursor_pos != (int)g_utf8_strlen (text, -1))
 				wbcg->auto_completing = FALSE;
 
 		if (event)
@@ -561,7 +593,7 @@ auto_complete_matches (WorkbookControlGUI *wbcg)
 	if (!wbcg->auto_completing)
 		return FALSE;
 
-	if (!wbcg->auto_complete_text)
+	if (wbcg->auto_complete_text == NULL)
 		return FALSE;
 
 	equal = (strncmp (text, wbcg->auto_complete_text, strlen (text)) == 0);
@@ -589,7 +621,10 @@ wbcg_edit_ctor (WorkbookControlGUI *wbcg)
 	g_assert (IS_WORKBOOK_CONTROL_GUI (wbcg));
 	g_assert (wbcg->edit_line.entry == NULL);
 
-	wbcg->edit_line.entry = gnumeric_expr_entry_new (wbcg, FALSE);
+	wbcg->edit_line.entry = g_object_new (GNM_EXPR_ENTRY_TYPE,
+					      "with_icon", FALSE,
+					      "wbcg", wbcg,
+					      NULL);
 	wbcg->edit_line.temp_entry = NULL;
 	wbcg->edit_line.guru = NULL;
 	wbcg->edit_line.signal_changed = -1;
